#
# Use-R! - Meta-Analysis with R by Schwarzer, Carpenter, Rcker
#
# R code for Chapter 7
# "Multivariate Meta-Analysis"
#
# slightly modified by Guido Knapp
# Load the meta package
library(meta)
# Change working directory - replace 'my_directory' with the path to
# an existing directory containing the data files
# setwd("my_directory")
setwd("E:/Vellore-Stick/RPrograms")
#
# 1. Read in the data
data14 <- read.csv("dataset14.csv")
# 2. Print dataset
data14
# Univariate meta-analysis of the DAS-28 outcome
m.das <- metagen(mean.das, se.das,
                 data=data14, sm="MD",
                 studlab=paste(author, year),
                 comb.random=FALSE)
# Univariate meta-analysis of the HAQ outcome
m.haq <- metagen(mean.haq, se.haq,
                 data=data14, sm="MD",
                 studlab=paste(author, year),
                 comb.random=FALSE)
#
# Forest plot for DAS-28
forest(m.das, hetstat=FALSE)
#
# Forest plot for HAQ
forest(m.haq, hetstat=FALSE)
#
# Load the mvmeta package
library(mvmeta)
#
# Arguments if the R function mvmeta
args(mvmeta)
#
# Function for calculating the covariance
# given standard deviations correlation
cor2cov <- function(sd1, sd2, cor) sd1*sd2*cor
#
#
#Combine the two outcomes into a matrix
theta <- cbind(data14$mean.das, data14$mean.haq)
# Add row and column names
# Rows and columns are seperated by "," in the list
dimnames(theta) <- list(data14$author, c("mean.das", "mean.haq"))
#
# Give a value for the unknown correlation
rho <- 0
# Information of the variances and covariance
# Note the order: Variance of the first outcome,
# covariance (computed with the above function cor2cov),
# variance of the second outcome
S.arth <- cbind(data14$se.das^2,
                cor2cov(data14$se.das, data14$se.haq, rho),
                data14$se.haq^2)
# Again row and column names
dimnames(S.arth) <- list(data14$author,
                         c("var.das", "cov", "var.haq"))
# the object looks like this now
S.arth
#
# Multivariate fixed effect meta-analysis
m.arth <- mvmeta(theta, S.arth, method="fixed")
print(summary(m.arth), digits=2)
#
# All studies have a correlation of 0.9 between outcomes
rho <- 0.9
S.arth2 <- cbind(data14$se.das^2,
                 cor2cov(data14$se.das, data14$se.haq, rho),
                 data14$se.haq^2)
dimnames(S.arth2) <- dimnames(S.arth)
# All studies have a correlation of -0.9 between outcomes
rho <- -0.9
S.arth3 <- cbind(data14$se.das^2,
                 cor2cov(data14$se.das, data14$se.haq, rho),
                 data14$se.haq^2)
dimnames(S.arth3) <- dimnames(S.arth)
# A mix of modest positive correlations between outcomes
rho <- c(0.5, 0.2, 0.1, 0.6, 0.4)
S.arth4 <- cbind(data14$se.das^2,
                 cor2cov(data14$se.das, data14$se.haq, rho),
                 data14$se.haq^2)
dimnames(S.arth4) <- dimnames(S.arth)
# Do meta-analyses
m.arth2 <- mvmeta(theta, S.arth2, method="fixed")
m.arth3 <- mvmeta(theta, S.arth3, method="fixed")
m.arth4 <- mvmeta(theta, S.arth4, method="fixed")
#
# Fixed effect means
round(coef(m.arth2), 3)
# Covariance matrix
vcov(m.arth2)
# Standard errors of fixed effect means
round(sqrt(diag(vcov(m.arth2))), 4)
#
# Funnel plots of the two outcomes
oldpar <- par(mfrow=c(1,2), mar=c(4,4.1,1,1.8))
#
funnel(m.das,
       xlab="Mean difference in DAS-28 score from baseline")
sel <- m.das$studlab %in% c("Bennet 2005", "Bombardieri 2007",
                            "Navarro-Sarabia 2009")
text(m.das$TE[sel]-0.015, m.das$seTE[sel]-0.002,
     m.das$studlab[sel],
     adj=1, cex=0.8)
text(m.das$TE[!sel]+0.015, m.das$seTE[!sel]-0.002,
     m.das$studlab[!sel],
     adj=0, cex=0.8)
funnel(m.haq,
       xlab="Mean difference in HAQ score from baseline")
text(m.haq$TE-0.005, m.haq$seTE-0.002,
     m.haq$studlab,
     adj=1, cex=0.8)
par(oldpar)
#
# Load R library ellipse which provides R function ellipse
library(ellipse)
# Plot the study means, setting the x-limits and y-limits
# so that the confidence regions will be visible
plot(data14$mean.das, data14$mean.haq,
     xlim=c(-2.5, -0.5), ylim=c(-0.7, 0), pch="+",
     xlab="Mean difference in DAS-28 score from baseline",
     ylab="Mean difference in HAQ score from baseline")
# Add confidence regions
rho <- 0.25
with(data14,
     for (i in seq(along=mean.das)){
       S <- matrix(c(se.das[i]^2,
                     se.das[i]*se.haq[i]*rho,
                     se.das[i]*se.haq[i]*rho,
                     se.haq[i]^2),
                   byrow=TRUE, ncol=2)
       lines(ellipse(S, centre=c(mean.das[i], mean.haq[i]),
                     level=0.95 ), col="grey")
     })
# Add study labels
studlab <- paste(data14$author, data14$year)
# Study label from Van der Bijl needs a line break
# in order not to overwrite another label
bijl <- data14$author=="Van der Bijl"
studlab[bijl] <- paste(data14$author[bijl], "\n", data14$year[bijl])
text(data14$mean.das+0.015, data14$mean.haq, studlab, adj=0)
#
#
# Multivariate Random Effects Meta-Analysis
rho <- 0.25
S.arth.r <- cbind(data14$se.das^2,
                  cor2cov(data14$se.das, data14$se.haq, rho),
                  data14$se.haq^2)
m.arth.reml <- mvmeta(theta, S.arth.r, method="reml")
m.arth.ml   <- mvmeta(theta, S.arth.r, method="ml")
m.arth.mm   <- mvmeta(theta, S.arth.r, method="mm")
#
print(summary(m.arth.reml), digits=2)
                 byrow=TRUE, ncol=2)
#
