####################################################################
#
#  R code to reproduce the "Crins et al. (2014)" example
#  shown in Figure 1 in
#
#    T. Friede, C. Röver, S. Wandel, B. Neuenschwander.
#    Meta-analysis of two studies in the presence of heterogeneity
#    with applications in rare diseases.
#    Biometrical Journal, 2016
#
#  The data represent two randomized studies,
#  the endpoint is acute rejection (AR),
#  and the considered effect measure is the logarithmic odd ratio.
#
#  Reproducing the remaining panels of the figure works analogously
#  (data are shown in the figure).
#

#  Crins example figure for April-2017 InSPiRe meeting slides
#  showing THREE studies (SRR endpoint)
#  (evolved from k2CrinsEcample-3.R)


# load necessary libraries:
require("metafor")
require("bayesmeta")


knha <- function(y, sigma, tau)
# Compute "normal approximation", "Hartung-Knapp-Sidik-Jonkman"
# and "modified Knapp-Hartung" 95% effect size confidence intervals
# (conditional on a given tau value)
#
# Parameters:
#   y     :  vector of effect sizes
#   sigma :  vector of corresponding standard errors
#   tau   :  estimated heterogeneity (standard deviation) value
{
  # sanity check:
  stopifnot(length(sigma)==length(y), all(sigma>0),
            length(tau)==1, tau>=0)
  # number of studies:
  k <- length(y)  
  # inverse variance weights:
  wi <- 1 / (sigma^2 + tau^2)
  # effect estimate:
  yHat <- sum(y*wi) / sum(wi)
  # ("usual") standard error:
  stderr <- sqrt(1/sum(wi))
  # "normal approximation" CI:
  normCI <- yHat + c(-1,1) * stderr * qnorm(0.975)
  # quadratic form "Q2":
  Q2 <- sum((y-yHat)^2 * wi) / (k-1)
  # Hartung-Knapp-Sidik-Jonkman interval:
  tquantile <- qt(0.975, df=(k-1))
  KnHaCI.1 <- yHat + c(-1,1) * stderr * sqrt(Q2) * tquantile
  # modified Knapp-Hartung interval (with "ad-hoc adjustment" for Q):
  Q2 <- max(c(Q2, 1))
  KnHaCI.2 <- yHat + c(-1,1) * stderr * sqrt(Q2) * tquantile
  names(normCI) <- names(KnHaCI.1) <- names(KnHaCI.2) <- c("lower","upper")
  result <- rbind("normal" = normCI,    # normal approximation
                  "HKSJ"   = KnHaCI.1,  # Hartung-Knapp-Sidik-Jonkman
                  "mKH"    = KnHaCI.2)  # modified Knapp-Hartung
  return(result)
}


# load data:
data("CrinsEtAl2014")

# compute effect sizes (log-OR for AR in randomized studies)
# using metafor library's "escalc()" function:
effsize <- escalc(measure = "OR",
                  ai = exp.SRR.events,  n1i = exp.total,
                  ci = cont.SRR.events, n2i = cont.total,
                  slab = publication,
                  subset = is.finite(CrinsEtAl2014[,"exp.SRR.events"]),
                  data = CrinsEtAl2014)

# compute Bayesian meta-analyses:
bm.hnorm10 <- bayesmeta(effsize, tau.prior = function(x){dhalfnormal(x,scale=1.00)})
bm.hnorm05 <- bayesmeta(effsize, tau.prior = function(x){dhalfnormal(x,scale=0.50)})

# compute frequentist (DerSimonian-Laird) meta-analysis:
rma.REML     <- rma.uni(yi=effsize$yi, vi=effsize$vi, method="REML")
print(rma.REML)
summary(rma.REML)

# compute corresponding 3 different CIs:
knhaCIs    <- knha(y=effsize$yi, sigma=sqrt(effsize$vi), tau=sqrt(rma.REML$tau2))
# (note: HKSJ and mKH are identical in this particular example)

# assemble estimates (on log-OR scale);
# estimates for tau and mu (posterior medians for Bayesian analyses)
# and 95% interval for mu:
estimates <- rbind("half-normal(1.0)" = c(bm.hnorm10$summary[2,1:2],   bm.hnorm10$summary[5:6,2]),
                   "half-normal(0.5)" = c(bm.hnorm05$summary[2,1:2],   bm.hnorm05$summary[5:6,2]),
                   "normal"     = c(sqrt(rma.REML$tau2), rma.REML$b, knhaCIs[1,]),
                   "HKSJ"       = c(sqrt(rma.REML$tau2), rma.REML$b, knhaCIs[2,]),
                   "mKH"        = c(sqrt(rma.REML$tau2), rma.REML$b, knhaCIs[3,]))
print(estimates)

# show forest plot:
fnames <- paste("/home/christian/lehre/2017-04-InSPiRe/CrinsExample0",1:2,".pdf",sep="")
for (i in 1:2) {
  pdf(file=fnames[i], paper="special", height=ifelse(i==1,3.2,4), width=6)
  par(mar=c(4,0,0,0)+0.1)
    if (i==1) {
      yrange <- c(-5 - nrow(effsize), 1)
    } else {
      yrange <- c(-7 - nrow(effsize), 1)
    }
    forest.default(effsize$yi, vi = effsize$vi, refline = 0,
                   rows = seq(-2, -length(effsize$yi) - 1, by = -1),
                   xlim = c(-9,7), ylim = yrange,
                   xlab="odds ratio", slab = effsize[,"publication"],
                   atransf=exp, at=log(c(0.01, 0.1, 1, 10)), digits=3)
    axis(1, at=log(c(0.02, 0.05,0.2,0.5,2,5)), lab=FALSE, tcl=-0.2)
    title("Crins et al. (2014) example: steroid-resistant graft rejection", line=-1.5)
    colvec <- c("red","green","blue","blue","blue")
    if (i==1) {
      for (j in 1:3)
        addpoly(estimates[j+2,"mu"], ci.lb=estimates[j+2,3], ci.ub=estimates[j+2,4], atransf=exp,
                mlab=row.names(estimates)[j+2], rows=yrange[1]+3.5-j, col=colvec[j+2], digits=3)
    } else {
      for (j in 1:nrow(estimates))
        addpoly(estimates[j,"mu"], ci.lb=estimates[j,3], ci.ub=estimates[j,4], atransf=exp,
                mlab=row.names(estimates)[j], rows=yrange[1]+5.5-j, col=colvec[j], digits=3)
    }
  par(mar=c(5,4,4,2)+0.1)
  dev.off()
}
